@@ -6,13 +6,10 @@
 
     Main script to start and set up OpenSlides.
 
-    :copyright: 2011, 2012 by OpenSlides team, see AUTHORS.
+    :copyright: 2011–2013 by OpenSlides team, see AUTHORS.
     :license: GNU GPL, see LICENSE for more details.
 """
 
-# for python 2.5 support
-from __future__ import with_statement
-
 import base64
 import ctypes
 import optparse
@@ -24,10 +21,12 @@ import threading
 import time
 import webbrowser
 
-import django.conf
+from django.conf import ENVIRONMENT_VARIABLE
 from django.core.management import execute_from_command_line
 
 from openslides import get_version
+from openslides.utils.tornado_webserver import run_tornado
+
 
 CONFIG_TEMPLATE = """#!/usr/bin/env python
 # -*- coding: utf-8 -*-
@@ -65,6 +64,11 @@ INSTALLED_PLUGINS = (
 )
 
 INSTALLED_APPS += INSTALLED_PLUGINS
+
+# Absolute path to the directory that holds media.
+# Example: "/home/media/media.lawrence.com/"
+MEDIA_ROOT = %(media_root_path)s
+
 """
 
 KEY_LENGTH = 30
@@ -74,33 +78,56 @@ KEY_LENGTH = 30
 _portable_db_path = object()
 
 
-def process_options(argv=None):
+def process_options(argv=None, manage_runserver=False):
     if argv is None:
         argv = sys.argv[1:]
 
     parser = optparse.OptionParser(
-        description="Run openslides using django's builtin webserver")
-    parser.add_option("-a", "--address", help="IP Address to listen on.")
-    parser.add_option("-p", "--port", type="int", help="Port to listen on.")
+        description="Run openslides using the tornado webserver")
+    parser.add_option(
+        "-a", "--address",
+        help="IP Address to listen on. Default: 0.0.0.0")
+    parser.add_option(
+        "-p", "--port", type="int",
+        help="Port to listen on. Default: 8000 (start as admin/root: 80)")
     parser.add_option(
         "--syncdb", action="store_true",
         help="Update/create database before starting the server.")
+    parser.add_option(
+        "--backupdb", action="store", metavar="BACKUP_PATH",
+        help="Make a backup copy of the database to BACKUP_PATH.")
     parser.add_option(
         "--reset-admin", action="store_true",
         help="Make sure the user 'admin' exists and uses 'admin' as password.")
     parser.add_option(
-        "-s", "--settings", help="Path to the openslides configuration.")
+        "-s", "--settings", help="Set the path to the settings file.")
+    parser.add_option(
+        "--no-browser",
+        action="store_false", dest="start_browser", default=True,
+        help="Do not automatically start the web browser.")
     parser.add_option(
         "--no-reload", action="store_true",
-        help="Do not reload the development server.")
+        help="Do not reload the web server.")
+    parser.add_option(
+        "--no-run", action="store_true",
+        help="Do not start the web server.")
     parser.add_option(
         "--version", action="store_true",
         help="Show version and exit.")
 
     opts, args = parser.parse_args(argv)
+
+    # Do not parse any argv if the script is started via manage.py runserver.
+    # This simulates the django runserver command
+    if manage_runserver:
+        opts.start_browser = False
+        opts.no_reload = False
+        return opts
+
     if opts.version:
         print get_version()
         exit(0)
+
     if args:
         sys.stderr.write("This command does not take arguments!\n\n")
         parser.print_help()
@@ -109,8 +136,8 @@ def process_options(argv=None):
     return opts
 
 
-def main(argv=None):
-    opts = process_options(argv)
+def main(argv=None, manage_runserver=False):
+    opts = process_options(argv, manage_runserver)
     _main(opts)
 
 
@@ -155,27 +182,35 @@ def _main(opts, database_path=None):
 
     # Find url to openslides
     addr, port = detect_listen_opts(opts.address, opts.port)
-    if port == 80:
-        url = "http://%s" % addr
-    else:
-        url = "http://%s:%d" % (addr, port)
 
     # Create Database if necessary
     if not database_exists() or opts.syncdb:
         run_syncdb()
-        set_system_url(url)
-        create_or_reset_admin_user()
 
     # Reset Admin
     elif opts.reset_admin:
-        create_or_reset_admin_user()
+        reset_admin_user()
+
+    if opts.backupdb:
+        backup_database(opts.backupdb)
+
+    if opts.no_run:
+        return
 
     # Start OpenSlides
+    reload = True
     if opts.no_reload:
-        extra_args = ['--noreload']
-    else:
-        extra_args = []
-    start_openslides(addr, port, start_browser_url=url, extra_args=extra_args)
+        reload = False
+
+    if opts.start_browser:
+        if port == 80:
+            suffix = ""
+        else:
+            suffix = ":%d" % port
+        start_browser("http://localhost%s" % suffix)
+
+    # Start the server
+    run_tornado(addr, port, reload)
 
 
 def create_settings(settings_path, database_path=None):
@@ -184,14 +219,17 @@ def create_settings(settings_path, database_path=None):
     if database_path is _portable_db_path:
         database_path = get_portable_db_path()
         dbpath_value = 'openslides.main.get_portable_db_path()'
+        media_root_path_value = 'openslides.main.get_portable_media_root_path()'
     else:
         if database_path is None:
             database_path = get_user_data_path('openslides', 'database.sqlite')
         dbpath_value = repr(fs2unicode(database_path))
+        media_root_path_value = repr(fs2unicode(get_user_data_path('openslides', 'media', '')))
 
     settings_content = CONFIG_TEMPLATE % dict(
         default_key=base64.b64encode(os.urandom(KEY_LENGTH)),
-        dbpath=dbpath_value)
+        dbpath=dbpath_value,
+        media_root_path=media_root_path_value)
 
     if not os.path.exists(settings_module):
         os.makedirs(settings_module)
@@ -211,15 +249,12 @@ def setup_django_environment(settings_path):
         sys.exit(1)
     settings_module_dir = os.path.dirname(settings_path)
     sys.path.append(settings_module_dir)
-    os.environ[django.conf.ENVIRONMENT_VARIABLE] = '%s' % settings_module_name
+    os.environ[ENVIRONMENT_VARIABLE] = '%s' % settings_module_name
 
 
-def detect_listen_opts(address, port):
+def detect_listen_opts(address=None, port=None):
     if address is None:
-        try:
-            address = socket.gethostbyname(socket.gethostname())
-        except socket.error:
-            address = "127.0.0.1"
+        address = "0.0.0.0"
 
     if port is None:
         # test if we can use port 80
@@ -260,46 +295,17 @@ def run_syncdb():
     # now initialize the database
     argv = ["", "syncdb", "--noinput"]
     execute_from_command_line(argv)
-    execute_from_command_line(["", "loaddata", "groups_de"])
 
 
-def set_system_url(url):
+def reset_admin_user():
     # can't be imported in global scope as it already requires
     # the settings module during import
-    from openslides.config.models import config
-
-    key = "participant_pdf_system_url"
-    if key in config:
-        return
-    config[key] = url
+    from openslides.participant.api import create_or_reset_admin_user
+    create_or_reset_admin_user()
 
 
-def create_or_reset_admin_user():
-    # can't be imported in global scope as it already requires
-    # the settings module during import
-    from openslides.participant.models import User
-    try:
-        admin = User.objects.get(username="admin")
-        print("Password for user admin was reset to 'admin'")
-    except User.DoesNotExist:
-        admin = User()
-        admin.username = 'admin'
-        admin.last_name = 'Administrator'
-        print("Created default admin user")
-
-    admin.is_superuser = True
-    admin.default_password = 'admin'
-    admin.set_password(admin.default_password)
-    admin.save()
-
-
-def start_openslides(addr, port, start_browser_url=None, extra_args=[]):
-    argv = ["", "runserver", '--noreload'] + extra_args
-
-    argv.append("%s:%d" % (addr, port))
-
-    if start_browser_url:
-        start_browser(start_browser_url)
+def backup_database(dest_path):
+    argv = ["", "backupdb", "--destination={0}".format(dest_path)]
     execute_from_command_line(argv)
 
 
@@ -342,13 +348,25 @@ def get_user_data_path(*args):
     return os.path.join(fs2unicode(data_home), *args)
 
 
+def is_portable():
+    """Return True if openslides is run as portable version"""
+
+    # NOTE: sys.executable is the path of the *interpreter*
+    #       the portable version embeds python so it *is* the interpreter.
+    #       The wrappers generated by pip and co. will spawn
+    #       the usual python(w).exe, so there is no danger of mistaking
+    #       them for the portable even though they may also be called
+    #       openslides.exe
+    exename = os.path.basename(sys.executable).lower()
+    return exename == "openslides.exe"
+
+
 def get_portable_path(*args):
     # NOTE: sys.executable will be the path to openslides.exe
     #       since it is essentially a small wrapper that embeds the
     #       python interpreter
 
-    exename = os.path.basename(sys.executable).lower()
-    if exename != "openslides.exe":
+    if not is_portable():
         raise Exception(
             "Cannot determine portable path when "
             "not running as portable")
@@ -361,6 +379,10 @@ def get_portable_db_path():
     return get_portable_path('openslides', 'database.sqlite')
 
 
+def get_portable_media_root_path():
+    return get_portable_path('openslides', 'media', '')
+
+
 def win32_get_app_data_path(*args):
     shell32 = ctypes.WinDLL("shell32.dll")
     SHGetFolderPath = shell32.SHGetFolderPathW
@@ -381,4 +403,7 @@ def win32_get_app_data_path(*args):
 
 
 if __name__ == "__main__":
-    main()
+    if is_portable():
+        win32_portable_main()
+    else:
+        main()
