@@ -15,9 +15,10 @@ from time import time
 from django.dispatch import receiver
 from django.template.loader import render_to_string
 
-from openslides.config.models import config
+from openslides.config.api import config
+from django.template import RequestContext
+from openslides.utils.exceptions import OpenSlidesError
 
-from openslides.projector.signals import projector_overlays
 
 SLIDE = {}
 
@@ -114,7 +115,7 @@ class Widget(object):
     """
     Class for a Widget for the Projector-Tab.
     """
-    def __init__(self, name, html=None, template=None, context={},
+    def __init__(self, request, name, html=None, template=None, context={},
                  permission_required=None, display_name=None, default_column=1):
         self.name = name
         if display_name is None:
@@ -125,7 +126,12 @@ class Widget(object):
         if html is not None:
             self.html = html
         elif template is not None:
-            self.html = render_to_string(template, context)
+            self.html = render_to_string(
+                template_name=template,
+                dictionary=context,
+                context_instance=RequestContext(request))
+        else:
+            raise OpenSlidesError('A Widget must have either a html or a template argument.')
 
         self.permission_required = permission_required
         self.default_column = default_column
@@ -146,35 +152,28 @@ class Widget(object):
         return unicode(self.display_name)
 
 
-@receiver(projector_overlays, dispatch_uid="projector_countdown")
-def countdown(sender, **kwargs):
-    name = 'Countdown'
-    if kwargs['register']:
-        return name
-    if name in kwargs['call']:
-        if config['countdown_state'] == 'active':
-            seconds = max(0, int(config['countdown_start_stamp'] + config['countdown_time'] - time()))
-        elif config['countdown_state'] == 'paused':
-            seconds = max(0, int(config['countdown_start_stamp'] + config['countdown_time'] - config['countdown_pause_stamp']))
-        elif config['countdown_state'] == 'inactive':
-            seconds = max(0, int(config['countdown_time']))
-        else:
-            seconds = 0
+class Overlay(object):
+    """
+    Represents an overlay which can be seen on the projector.
+    """
 
-        if seconds == 0:
-            config['countdown_state'] = 'expired'
+    def __init__(self, name, get_widget_html, get_projector_html):
+        self.name = name
+        self.widget_html_callback = get_widget_html
+        self.projector_html_callback = get_projector_html
 
-        return (name, '%02d:%02d' % (seconds / 60, seconds % 60))
-    return None
+    def get_widget_html(self):
+        return self.widget_html_callback()
 
+    def get_projector_html(self):
+        try:
+            return self._projector_html
+        except AttributeError:
+            self._projector_html = self.projector_html_callback()
+            return self.get_projector_html()
 
-@receiver(projector_overlays, dispatch_uid="projector_message")
-def projector_message(sender, **kwargs):
-    name = 'Message'
-    if kwargs['register']:
-        return name
-    if name in kwargs['call']:
-        message = config['projector_message']
-        if message != '':
-            return (name, message)
-    return None
+    def is_active(self):
+        return self.name in config['projector_active_overlays']
+
+    def show_on_projector(self):
+        return self.is_active() and self.get_projector_html() is not None
